
#include "sys.h"
#include <stdlib.h>

//[of]:newEnvironmentVariable
//[c]Creates a new string with the value of the given environment variable.
//[c]
//[c]PARAMETERS
//[c]	name	--	The name of the variable
//[c]	out	--	The string created. Note that it is not a SysString but a String: 
//[c]			it must not be deleted with csys_deleteString(), but with the 
//[c]			regular String.delete method instead.
//[c]
//[c]RETURN VALUE
//[c]	An error code. It can be an error because of encoding or something else
//[c]	on Windows.
//[c]
ENTRY SysErrorCode csys_newEnvironmentVariable(const String name, String* out)
{
	#ifdef WINDOWS
		DWORD res = GetEnvironmentVariable(name, NULL, 0);
		if( res == 0 )
		{
			wchar* str;
			
			if( GetLastError() != ERROR_ENVVAR_NOT_FOUND )
				return ERR_SYSTEM;

			str = (wchar*)csys_allocate(1 * sizeof(wchar));
			*str = 0;
			*out = str;
		}
		else
		{
			// Note: the string should be validated (it must not contain invalid UTF-16 sequence)
			wchar* str = (wchar*)csys_allocate(res * sizeof(wchar));
			GetEnvironmentVariable(name, str, res);
			*out = str;
		}
		return OK;
	#else
		// As the name should contain only upper ASCII letter, digit or 
		// underscore, we don't convert the name to the current locale.
		char* value = getenv(name);
		if( value == NULL )
			value = "";
		return csys_newCStringFromLocale(out, value);
	#endif
} 
//[cf]
//[of]:hasEnvironmentVariable
ENTRY int csys_hasEnvironmentVariable(const String name)
{
	#ifdef WINDOWS
		return GetEnvironmentVariable(name, NULL, 0) != 0;
	#else
		// As the name should contain only upper ASCII letter, digit or 
		// underscore, we don't convert the name to the current locale.
		return getenv(name) != NULL;
	#endif
}
//[cf]
